@@ -34,7 +34,7 @@
 #include <glade/glade.h>
 
 #ifdef USE_HAL
-#  include <hal/libhal.h>
+# include <libhal.h>
 #endif
 
 #include "gfloppy.h"
