@@ -47,8 +47,8 @@
 #include "gdict-about.h"
 #include "gdict-window.h"
 
-#define GDICT_WINDOW_COLUMNS      52
-#define GDICT_WINDOW_ROWS         30
+#define GDICT_WINDOW_COLUMNS      56
+#define GDICT_WINDOW_ROWS         33
 
 #define GDICT_WINDOW_MIN_WIDTH	  400
 #define GDICT_WINDOW_MIN_HEIGHT	  330
@@ -60,6 +60,7 @@ enum
   PROP_SOURCE_LOADER,
   PROP_SOURCE_NAME,
   PROP_PRINT_FONT,
+  PROP_DEFBOX_FONT,
   PROP_WORD,
   PROP_WINDOW_ID
 };
@@ -121,6 +122,9 @@ gdict_window_finalize (GObject *object)
 
   if (window->notify_id)
     gconf_client_notify_remove (window->gconf_client, window->notify_id);
+
+  if (window->font_notify_id)
+    gconf_client_notify_remove (window->gconf_client, window->font_notify_id);
   
   if (window->gconf_client)
     g_object_unref (window->gconf_client);
@@ -147,6 +151,7 @@ gdict_window_finalize (GObject *object)
 
   g_free (window->source_name);
   g_free (window->print_font);
+  g_free (window->defbox_font);
   g_free (window->word);
   g_free (window->database);
   g_free (window->strategy);
@@ -299,6 +304,30 @@ get_context_from_loader (GdictWindow *window)
   return retval;
 }
 
+static void
+gdict_window_set_defbox_font (GdictWindow *window,
+			      const gchar *defbox_font)
+{
+  if (!defbox_font)
+    defbox_font = gconf_client_get_string (window->gconf_client,
+		    			   DOCUMENT_FONT_KEY,
+					   NULL);
+
+  if (!defbox_font)
+    defbox_font = GDICT_DEFAULT_DEFBOX_FONT;
+
+  if (window->defbox_font)
+    g_free (window->defbox_font);
+
+  window->defbox_font = g_strdup (defbox_font);
+
+  if (!window->defbox)
+    return;
+
+  gdict_defbox_set_font_name (GDICT_DEFBOX (window->defbox),
+		  	      window->defbox_font);
+}
+
 static void
 gdict_window_set_print_font (GdictWindow *window,
 			     const gchar *print_font)
@@ -321,11 +350,23 @@ static void
 gdict_window_set_word (GdictWindow *window,
 		       const gchar *word)
 {
+  gchar *title;
+  
   g_free (window->word);
   window->word = NULL;
 
   if (word && word[0] != '\0')
     window->word = g_strdup (word);
+
+  title = g_strdup_printf (_("%s - Dictionary"), window->word);
+  gtk_window_set_title (GTK_WINDOW (window), title);
+  g_free (title);
+  
+  window->max_definition = -1;
+  window->last_definition = 0;
+  
+  if (window->defbox)
+    gdict_defbox_lookup (GDICT_DEFBOX (window->defbox), window->word);
 }
 
 static void
@@ -416,6 +457,9 @@ gdict_window_set_property (GObject      *object,
     case PROP_PRINT_FONT:
       gdict_window_set_print_font (window, g_value_get_string (value));
       break;
+    case PROP_DEFBOX_FONT:
+      gdict_window_set_defbox_font (window, g_value_get_string (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -444,6 +488,9 @@ gdict_window_get_property (GObject    *object,
     case PROP_PRINT_FONT:
       g_value_set_string (value, window->print_font);
       break;
+    case PROP_DEFBOX_FONT:
+      g_value_set_string (value, window->defbox_font);
+      break;
     case PROP_WINDOW_ID:
       g_value_set_uint (value, window->window_id);
       break;
@@ -546,7 +593,6 @@ gdict_window_cmd_file_print (GtkAction   *action,
   gdict_show_print_dialog (GTK_WINDOW (window),
   			   _("Print"),
   			   GDICT_DEFBOX (window->defbox));
-
 }
 
 static void
@@ -854,6 +900,13 @@ gdict_window_gconf_notify_cb (GConfClient *client,
       else
         gdict_window_set_database (window, GDICT_DEFAULT_DATABASE);
     }
+  else if (strcmp (entry->key, DOCUMENT_FONT_KEY) == 0)
+    {
+      if (entry->value && (entry->value->type == GCONF_VALUE_STRING))
+        gdict_window_set_defbox_font (window, gconf_value_get_string (entry->value));
+      else
+        gdict_window_set_defbox_font (window, GDICT_DEFAULT_DEFBOX_FONT);
+    }
 }
 
 static void
@@ -873,15 +926,6 @@ entry_activate_cb (GtkWidget   *widget,
     return;
 
   gdict_window_set_word (window, word);
-  
-  title = g_strdup_printf (_("%s - Dictionary"), window->word);
-  gtk_window_set_title (GTK_WINDOW (window), title);
-  g_free (title);
-  
-  window->max_definition = -1;
-  window->last_definition = 0;
-  
-  gdict_defbox_lookup (GDICT_DEFBOX (window->defbox), window->word);
 }
 
 static void
@@ -905,19 +949,9 @@ gdict_window_drag_data_received_cb (GtkWidget        *widget,
       gtk_entry_set_text (GTK_ENTRY (window->entry), text);
 
       gdict_window_set_word (window, text);
-      
-      title = g_strdup_printf (_("%s - Dictionary"), window->word);
-      gtk_window_set_title (GTK_WINDOW (window), title);
-      g_free (title);
-      
-      window->max_definition = -1;
-      window->last_definition = 0;
-
-      gdict_defbox_lookup (GDICT_DEFBOX (window->defbox), window->word);
-
-      gtk_drag_finish (context, TRUE, FALSE, time_);
-      
       g_free (text);
+      
+      gtk_drag_finish (context, TRUE, FALSE, time_);
     }
   else
     gtk_drag_finish (context, FALSE, FALSE, time_);
@@ -941,18 +975,22 @@ gdict_window_size_allocate (GtkWidget     *widget,
 }
 
 static void
-gdict_window_style_set (GtkWidget *widget,
-			GtkStyle  *old_style)
+set_window_default_size (GdictWindow *window)
 {
-  GdictWindow *window;
-  gint width, height;
+  GtkWidget *widget;
   gboolean is_maximized;
+  gint width, height;
+  gint font_size;
+  GdkScreen *screen;
+  gint monitor_num;
+  GtkRequisition req;
+  GdkRectangle monitor;
 
-  if (GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set)
-    GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set (widget, old_style);
- 
-  window = GDICT_WINDOW (widget);
+  g_assert (GDICT_IS_WINDOW (window));
 
+  widget = GTK_WIDGET (window);
+  
+  /* recover the state from GConf */
   width = gconf_client_get_int (window->gconf_client,
 		  		GDICT_GCONF_WINDOW_WIDTH_KEY,
 				NULL);
@@ -963,8 +1001,8 @@ gdict_window_style_set (GtkWidget *widget,
 		  			GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
 					NULL);
   
-  /* the user wants gnome-dictionary to resize itself, so we
-   * compute the minimum safe geometry needed for displaying
+  /* XXX - the user wants gnome-dictionary to resize itself, so
+   * we compute the minimum safe geometry needed for displaying
    * the text returned by a dictionary server, which is based
    * on the font size and the ANSI terminal.  this is dumb,
    * I know, but dictionary servers return pre-formatted text
@@ -972,18 +1010,30 @@ gdict_window_style_set (GtkWidget *widget,
    */
   if (width == -1 || height == -1)
     {
-      PangoContext *pango_context;
-      PangoFontDescription *font_desc;
-      gint font_size;
+      /* Size based on the font size */
+      GtkWidget *defbox = window->defbox;
       
-      pango_context = gtk_widget_get_pango_context (widget);
-      font_desc = pango_context_get_font_description (pango_context);
-      font_size = pango_font_description_get_size (font_desc) / PANGO_SCALE;
+      font_size = pango_font_description_get_size (defbox->style->font_desc);
+      font_size = PANGO_PIXELS (font_size);
 
-      width = MAX (GDICT_WINDOW_COLUMNS * font_size, GDICT_WINDOW_MIN_WIDTH);
-      height = MAX (GDICT_WINDOW_ROWS * font_size, GDICT_WINDOW_MIN_HEIGHT);
+      width = font_size * GDICT_WINDOW_COLUMNS;
+      height = font_size * GDICT_WINDOW_ROWS;
+
+      /* Use at least the requisition size of the window... */
+      gtk_widget_size_request (widget, &req);
+      width = MAX (width, req.width);
+      height = MAX (height, req.height);
+
+      /* ... but make it no larger than the monitor */
+      screen = gtk_widget_get_screen (widget);
+      monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+
+      gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+      width = MIN (width, monitor.width * 3 / 4);
+      height = MIN (height, monitor.height * 3 / 4);
     }
-  
+
+  /* Set default size */
   gtk_window_set_default_size (GTK_WINDOW (widget),
   			       width,
   			       height);
@@ -992,6 +1042,18 @@ gdict_window_style_set (GtkWidget *widget,
     gtk_window_maximize (GTK_WINDOW (widget));
 }
 
+static void
+gdict_window_style_set (GtkWidget *widget,
+			GtkStyle  *old_style)
+{
+  GdictWindow *window;
+
+  if (GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set)
+    GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set (widget, old_style);
+
+  set_window_default_size (GDICT_WINDOW (widget));
+}
+
 static GObject *
 gdict_window_constructor (GType                  type,
 			  guint                  n_construct_properties,
@@ -1006,6 +1068,8 @@ gdict_window_constructor (GType                  type,
   GtkWidget *label;
   GtkActionGroup *action_group;
   GtkAccelGroup *accel_group;
+  PangoFontDescription *font_desc;
+  gchar *font_name;
   GError *error;
   
   object = G_OBJECT_CLASS (gdict_window_parent_class)->constructor (type,
@@ -1014,46 +1078,6 @@ gdict_window_constructor (GType                  type,
   window = GDICT_WINDOW (object);
   
   gtk_widget_push_composite_child ();
-
-  /* retrieve the window state from gconf */
-  is_maximized = gconf_client_get_bool (window->gconf_client,
-		  			GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
-					NULL);
-
-  width = gconf_client_get_int (window->gconf_client,
-		  		GDICT_GCONF_WINDOW_WIDTH_KEY,
-				NULL);
-  height = gconf_client_get_int (window->gconf_client,
-		  		 GDICT_GCONF_WINDOW_HEIGHT_KEY,
-				 NULL);
-  if (width == -1 || height == -1)
-    {
-      PangoContext *pango_context;
-      PangoFontDescription *font_desc;
-      gint font_size;
-  
-      pango_context = gtk_widget_get_pango_context (GTK_WIDGET (window));
-      font_desc = pango_context_get_font_description (pango_context);
-
-      font_size = pango_font_description_get_size (font_desc) / PANGO_SCALE;
-
-      width = MAX (GDICT_WINDOW_COLUMNS * font_size, GDICT_WINDOW_MIN_WIDTH);
-      height = MAX (GDICT_WINDOW_ROWS * font_size, GDICT_WINDOW_MIN_HEIGHT);
-    }
-  else
-    {
-      window->default_width = width;
-      window->default_height = height;
-    }
-
-  window->is_maximized = is_maximized;
-  
-  gtk_window_set_title (GTK_WINDOW (window), _("Dictionary"));
-  gtk_window_set_default_size (GTK_WINDOW (window),
-  			       width,
-  			       height);
-  if (is_maximized)
-    gtk_window_maximize (GTK_WINDOW (window));
  
   window->main_box = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), window->main_box);
@@ -1131,6 +1155,58 @@ gdict_window_constructor (GType                  type,
   gtk_box_pack_end (GTK_BOX (window->main_box), window->status, FALSE, FALSE, 0);
   gtk_widget_show (window->status);
 
+  /* retrieve the font size from gconf */
+  font_name = gconf_client_get_string (window->gconf_client,
+		  		       DOCUMENT_FONT_KEY,
+				       NULL);
+  if (!font_name)
+    font_name = GDICT_DEFAULT_DEFBOX_FONT;
+  
+  gdict_window_set_defbox_font (window, font_name);
+  font_desc = pango_font_description_from_string (font_name);
+  
+  /* retrieve the window state from gconf */
+  is_maximized = gconf_client_get_bool (window->gconf_client,
+		  			GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
+					NULL);
+
+  width = gconf_client_get_int (window->gconf_client,
+		  		GDICT_GCONF_WINDOW_WIDTH_KEY,
+				NULL);
+  height = gconf_client_get_int (window->gconf_client,
+		  		 GDICT_GCONF_WINDOW_HEIGHT_KEY,
+				 NULL);
+  
+  /* if the (width, height) tuple is not defined, use the font to
+   * calculate the right window geometry
+   */
+  if (width == -1 || height == -1)
+    {
+      gint font_size;
+  
+      font_size = pango_font_description_get_size (font_desc);
+      font_size = PANGO_PIXELS (font_size);
+
+      width = MAX (GDICT_WINDOW_COLUMNS * font_size, GDICT_WINDOW_MIN_WIDTH);
+      height = MAX (GDICT_WINDOW_ROWS * font_size, GDICT_WINDOW_MIN_HEIGHT);
+    }
+  else
+    {
+      window->default_width = width;
+      window->default_height = height;
+    }
+
+  pango_font_description_free (font_desc);
+  
+  window->is_maximized = is_maximized;
+  
+  gtk_window_set_title (GTK_WINDOW (window), _("Dictionary"));
+  gtk_window_set_default_size (GTK_WINDOW (window),
+  			       width,
+  			       height);
+  if (is_maximized)
+    gtk_window_maximize (GTK_WINDOW (window));
+  
   g_signal_connect (window, "delete-event",
 		    G_CALLBACK (gdict_window_delete_event_cb),
 		    NULL);
@@ -1178,6 +1254,13 @@ gdict_window_class_init (GdictWindowClass *klass)
   				   			"The font name to be used when printing",
   				   			GDICT_DEFAULT_PRINT_FONT,
   				   			(G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT)));
+  g_object_class_install_property (gobject_class,
+		  		   PROP_DEFBOX_FONT,
+				   g_param_spec_string ("defbox-font",
+					   		"Defbox Font",
+							"The font name to be used by the defbox widget",
+							GDICT_DEFAULT_DEFBOX_FONT,
+							(G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT)));
   g_object_class_install_property (gobject_class,
 		  		   PROP_WORD,
 				   g_param_spec_string ("word",
@@ -1211,26 +1294,64 @@ gdict_window_init (GdictWindow *window)
 {
   gchar *icon_file;
   GdkPixbuf *icon;
-  GError *icon_error;
+  GError *icon_error, *gconf_error;
   
   window->loader = NULL;
   window->context = NULL;
-  
+
   window->gconf_client = gconf_client_get_default ();
+
+  gconf_error = NULL;
   gconf_client_add_dir (window->gconf_client,
   			GDICT_GCONF_DIR,
   			GCONF_CLIENT_PRELOAD_NONE,
-  			NULL);
+  			&gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+		         _("Unable to connect to GConf"),
+		         gconf_error->message);
+
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
+
   window->notify_id = gconf_client_notify_add (window->gconf_client,
   					       GDICT_GCONF_DIR,
   					       gdict_window_gconf_notify_cb,
   					       window,
   					       NULL,
-  					       NULL);
+  					       &gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+		         _("Unable to get notification for preferences"),
+		         gconf_error->message);
+
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
+
+  window->font_notify_id = gconf_client_notify_add (window->gconf_client,
+		  				    DOCUMENT_FONT_KEY,
+						    gdict_window_gconf_notify_cb,
+						    window,
+						    NULL,
+						    &gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+		         _("Unable to get notification for the document font"),
+		         gconf_error->message);
+
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
   
   window->word = NULL;
   window->source_name = NULL;
   window->print_font = NULL;
+  window->defbox_font = NULL;
 
   window->database = NULL;
   window->strategy = NULL;
@@ -1243,7 +1364,7 @@ gdict_window_init (GdictWindow *window)
   icon = gdk_pixbuf_new_from_file (icon_file, &icon_error);
   if (icon_error)
     {
-      show_error_dialog (GTK_WINDOW (window),
+      show_error_dialog (NULL,
 		         _("Unable to load the application icon"),
 		         icon_error->message);
 
@@ -1269,11 +1390,21 @@ gdict_window_new (GdictSourceLoader *loader,
 		  const gchar       *source_name,
 		  const gchar       *word)
 {
+  GtkWidget *retval;
+  
   g_return_val_if_fail (GDICT_IS_SOURCE_LOADER (loader), NULL);
   
-  return g_object_new (GDICT_TYPE_WINDOW,
-                       "source-loader", loader,
-		       "source-name", source_name,
-		       "word", word,
-                       NULL);
+  retval = g_object_new (GDICT_TYPE_WINDOW,
+                         "source-loader", loader,
+			 "source-name", source_name,
+			 NULL);
+
+  if (word && word[0] != '\0')
+    {
+      gtk_entry_set_text (GTK_ENTRY (GDICT_WINDOW (retval)->entry), word);
+      
+      gdict_window_set_word (GDICT_WINDOW (retval), word);
+    }
+
+  return retval;
 }
