@@ -46,7 +46,10 @@ extern "C" {
 #define GNOME_SEARCH_TOOL_ICON "gnome-searchtool"
 #define MINIMUM_WINDOW_WIDTH   420
 #define MINIMUM_WINDOW_HEIGHT  310
+#define DEFAULT_WINDOW_WIDTH   554
+#define DEFAULT_WINDOW_HEIGHT  350
 #define WINDOW_HEIGHT_STEP      35
+#define NUM_VISIBLE_COLUMNS      5
 
 typedef enum {
 	STOPPED,
@@ -66,6 +69,7 @@ typedef enum {
 	COLUMN_TYPE,
 	COLUMN_READABLE_DATE,
 	COLUMN_DATE,
+	COLUMN_MONITOR,
 	COLUMN_NO_FILES_FOUND,
 	NUM_COLUMNS
 } GSearchResultColumns;
@@ -74,6 +78,7 @@ typedef struct _GSearchWindow GSearchWindow;
 typedef struct _GSearchWindowClass GSearchWindowClass;
 typedef struct _GSearchCommandDetails GSearchCommandDetails;
 typedef struct _GSearchConstraint GSearchConstraint;
+typedef struct _GSearchMonitor GSearchMonitor;
 
 struct _GSearchWindow {
 	GtkWindow               parent_instance;
@@ -81,6 +86,9 @@ struct _GSearchWindow {
 	GtkWidget             * window;
 	GtkUIManager          * window_ui_manager;
 	GdkGeometry             window_geometry;
+	gint                    window_width;
+	gint                    window_height;
+	gboolean                is_window_maximized;
 	gboolean                is_window_accessible;
 	
 	GtkWidget             * name_contains_entry;
@@ -89,6 +97,7 @@ struct _GSearchWindow {
 	GtkWidget             * progress_spinner;
 	GtkWidget             * find_button;
 	GtkWidget             * stop_button;
+	GtkWidget             * focus;
 
 	GtkWidget             * show_more_options_expander;
 	GtkWidget             * available_options_vbox;
@@ -151,6 +160,12 @@ struct _GSearchWindowClass {
 	GtkWindowClass parent_class;
 };
 
+struct _GSearchMonitor {
+	GSearchWindow         * gsearch;
+	GtkTreeRowReference   * reference;
+	GnomeVFSMonitorHandle * handle;
+};
+
 GType 
 gsearch_window_get_type (void);
 
@@ -190,6 +205,12 @@ get_desktop_item_name (GSearchWindow * gsearch);
 void
 update_search_counts (GSearchWindow * gsearch);
 
+gboolean
+tree_model_iter_free_monitor (GtkTreeModel * model,
+                              GtkTreePath * path,
+                              GtkTreeIter * iter,
+                              gpointer data);
+
 #ifdef __cplusplus
 }
 #endif
