@@ -32,13 +32,9 @@
 #include "logrtns.h"
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include "misc.h"
+#include "math.h"
 
 char *error_main = N_("One file or more could not be opened");
-enum {
-    MONTH = 0, DAY,
-    HOUR, MIN, SEC,
-    HOSTNAME, PROCESS, MESSAGE
-};
 
 static LogStats *log_stats_new (char *filename, gboolean show_error);
 
@@ -49,7 +45,10 @@ file_exist (char *filename, gboolean show_error)
 {
    GnomeVFSHandle *handle;
    GnomeVFSResult result;
-   char buff[1024];
+   char *secondary = NULL;
+
+   if (filename == NULL)
+       return;
 
    result = gnome_vfs_open (&handle, filename, GNOME_VFS_OPEN_READ);
    if (result != GNOME_VFS_OK) {
@@ -57,19 +56,19 @@ file_exist (char *filename, gboolean show_error)
 		   switch (result) {
 		   case GNOME_VFS_ERROR_ACCESS_DENIED:
 		   case GNOME_VFS_ERROR_NOT_PERMITTED:
-			   g_snprintf (buff, sizeof (buff),
-				       _("%s is not user readable. "
+			   secondary = g_strdup_printf (_("%s is not user readable. "
 					 "Either run the program as root or ask the sysadmin to "
 					 "change the permissions on the file.\n"), filename);
 			   break;
 		   case GNOME_VFS_ERROR_TOO_BIG:
-			   g_snprintf (buff, sizeof (buff), _("%s is too big."), filename);
+			   secondary = g_strdup_printf (_("%s is too big."), filename);
 			   break;
 		   default:
-			   g_snprintf (buff, sizeof (buff),
-				       _("%s could not be opened."), filename);
+			   secondary = g_strdup_printf (_("%s could not be opened."), filename);
+               break;
 		   }
-		   ShowErrMessage (NULL, error_main, buff);
+		   error_dialog_show (NULL, error_main, secondary);
+           g_free (secondary);
 	   }
 	   return FALSE;
    }
@@ -83,20 +82,32 @@ file_is_zipped (char *filename)
 {
 	char *mime_type;
 
+    if (filename == NULL)
+        return;
+
 	mime_type = gnome_vfs_get_mime_type (filename);
+    if (mime_type == NULL)
+        return FALSE;
 
 	if (strcmp (mime_type, "application/x-gzip")==0 ||
 	    strcmp (mime_type, "application/x-zip")==0 ||
-	    strcmp (mime_type, "application/zip")==0)
+	    strcmp (mime_type, "application/zip")==0) {
+        g_free (mime_type);
 		return TRUE;
-	else
+    } else {
+        g_free (mime_type);
 		return FALSE;
+    }
 }
 
 gboolean
 file_is_log (char *filename, gboolean show_error)
 {
     LogStats *stats;
+
+    if (filename == NULL)
+        return;
+
     stats = log_stats_new (filename, show_error);
     if (stats==NULL)
         return FALSE;
@@ -106,68 +117,6 @@ file_is_log (char *filename, gboolean show_error)
     }
 }
 
-/* log line manipulations */
-
-
-void
-logline_fill_from_string (LogLine *line, gchar *message, gboolean has_date)
-{
-   char **splits;
-   int i, field;
-   
-   if (line == NULL)
-       return;
-
-   if (message[0] == 0) {
-       line->message=NULL;       
-       return;
-   }
-
-   if (!has_date) {
-       line->message = g_strdup (message);
-       return;
-   }
-
-   /* Seperation of a log line into 8 fields */
-   /* Some of them can be empty */
-   splits = g_strsplit_set (message, " :\n", 9);
-
-   i = 0; field= 0;
-   while (splits[i]!=NULL) {
-
-       if (g_str_equal (splits[i], ""))
-           i++;
-
-       switch (field) {
-       case HOUR:
-           line->hour = (int) atoi (splits[i]);
-           break;
-       case MIN:
-           line->min = (int) atoi (splits[i]);
-           break;
-       case SEC:
-           line->sec = (int) atoi (splits[i]);
-           break;
-       case HOSTNAME:
-           line->hostname = g_strdup (splits[i]);
-           break;
-       case PROCESS:
-           line->process = g_strdup (splits[i]);
-           break;
-       case MESSAGE:
-           line->message = g_strdup (splits[i]);
-           break;
-       default:
-           break;
-       }
-       i++;
-       field++;
-   }
-
-   g_strfreev (splits);
-   return;
-}
-
 /* log functions */
 
 gint 
@@ -184,8 +133,14 @@ string_get_date_string (gchar *line)
     gchar *month=NULL, *day=NULL;
     int i=0;
 
+    if (line == NULL || line[0] == 0)
+        return;
+
     split = g_strsplit (line, " ", 4);
-    while ((day == NULL || month == NULL) && split[i]!=NULL) {
+    if (split == NULL)
+        return;
+
+    while ((day == NULL || month == NULL) && split[i]!=NULL && i<4) {
         if (g_str_equal (split[i], "")) {
             i++;
             continue;
@@ -193,6 +148,10 @@ string_get_date_string (gchar *line)
 
         if (month == NULL) {
             month = split[i++];
+            /* If the first field begins by a number, the date
+               is given in yyyy-mm-dd format */
+            if (!g_ascii_isalpha (month[0]))
+                break;
             continue;
         }
 
@@ -209,23 +168,28 @@ string_get_date_string (gchar *line)
     return (date_string);
 }
 
-
 /* log_read_dates
    Read all dates which have a log entry to create calendar.
    All dates are given with respect to the 1/1/1970
    and are then corrected to the correct year once we
    reach the end.
 */
-GList *
+GSList *
 log_read_dates (gchar **buffer_lines, time_t current)
 {
    int offsetyear = 0, current_year;
-   GList *days = NULL, *days_copy;
+   GSList *days = NULL, *days_copy;
    GDate *date, *newdate;
    struct tm *tmptm;
    gchar *date_string;
    Day *day;
-   int i;
+   gboolean done = FALSE;
+   int i, n, rangemin, rangemax;
+
+   if (buffer_lines == NULL)
+       return NULL;
+
+   n = g_strv_length (buffer_lines);
 
    tmptm = localtime (&current);
    current_year = tmptm->tm_year + 1900;
@@ -233,6 +197,8 @@ log_read_dates (gchar **buffer_lines, time_t current)
    for (i=0; buffer_lines[i]==NULL; i++);
 
    /* Start building the list */
+   /* Scanning each line to see if the date changed is too slow, so we proceed
+      in a recursive fashion */
 
    date = string_get_date (buffer_lines[i]);
    if ((date==NULL)|| !g_date_valid (date))
@@ -240,55 +206,92 @@ log_read_dates (gchar **buffer_lines, time_t current)
 
    g_date_set_year (date, current_year);
    day = g_new (Day, 1);
-   days = g_list_append (days, day);
+   days = g_slist_append (days, day);
 
    day->date = date;
    day->first_line = i;
+   day->last_line = -1;
    date_string = string_get_date_string (buffer_lines[i]);
 
-   /* Count days appearing in the log */
+   rangemin = 0;
+   rangemax = n-1;
 
-   for (i=day->first_line; buffer_lines[i] != NULL; i++) {
-
-       if (g_str_has_prefix (buffer_lines[i], date_string))
-           continue;
-       g_free (date_string);
-       date_string = string_get_date_string (buffer_lines[i]);
-
-       newdate = string_get_date (buffer_lines[i]);
-       if (newdate == NULL)
-           continue;
+   while (!done) {
        
-       day->last_line = i-1;
-       
-       /* Append the day to the list */	
-       g_date_set_year (newdate, current_year + offsetyear);	
-       if (g_date_compare (newdate, date) < 1) {
-		   offsetyear++; /* newdate is next year */
-           g_date_add_years (newdate, 1);
+       i = n-1;
+       while (day->last_line < 0) {
+
+           if (g_str_has_prefix (buffer_lines[i], date_string)) {
+               if (i == (n-1)) {
+                   day->last_line = i;
+                   done = TRUE;
+                   break;
+               } else {
+                   if (!g_str_has_prefix (buffer_lines[i+1], date_string)) {
+                       day->last_line = i;
+                       break;
+                   } else {
+                       rangemin = i;
+                       i = floor ( ((float) i + (float) rangemax)/2.);
+                   }
+               }
+           } else {
+               rangemax = i;
+               i = floor (((float) rangemin + (float) i)/2.);               
+           }
+
        }
+       
+       g_free (date_string);
 
-       date = newdate;
-       day = g_new (Day, 1);
-       days = g_list_append (days, day);
+       if (!done) {
+         /* We need to find the first line now that has a date
+            Logs can have some messages without dates ... */
+         newdate = NULL;
+         while (newdate == NULL && !done) {
+           i++;
+           date_string = string_get_date_string (buffer_lines[i]);
+           if (date_string == NULL)
+             continue;
+           newdate = string_get_date (buffer_lines[i]);
+           
+           if (newdate == NULL && i==n-1)
+             done = TRUE;
+         }
 
-	   day->date = date;
-	   day->first_line = i;
+         day->last_line = i-1;
+
+         /* Append a day to the list */	
+         if (newdate) {
+           g_date_set_year (newdate, current_year + offsetyear);	
+           if (g_date_compare (newdate, date) < 1) {
+             offsetyear++; /* newdate is next year */
+             g_date_add_years (newdate, 1);
+           }
+           
+           date = newdate;
+           day = g_new (Day, 1);
+           days = g_slist_append (days, day);
+           
+           day->date = date;
+           day->first_line = i;
+           day->last_line = -1;
+           rangemin = i;
+           rangemax = n;
+         }
+       }
    }
-   day->last_line = i-1;
-   days = g_list_first (days);
-   g_free (date_string);
 
    /* Correct years now. We assume that the last date on the log
       is the date last accessed */
 
-   for (days_copy = days; days_copy != NULL; days_copy = g_list_next (days_copy)) {       
+   for (days_copy = days; days_copy != NULL; days_copy = g_slist_next (days_copy)) {       
        day = days_copy -> data;
        g_date_subtract_years (day->date, offsetyear);
    }
    
    /* Sort the days in chronological order */
-   days = g_list_sort (days, days_compare);
+   days = g_slist_sort (days, days_compare);
 
    return (days);
 }
@@ -306,10 +309,13 @@ log_stats_new (char *filename, gboolean show_error)
    GnomeVFSFileInfo *info;
    GnomeVFSHandle *handle;
    GnomeVFSFileSize size;
-   LogStats *lstats;
+   LogStats *stats;
    char buff[1024];
    char *found_space;
 
+   if (filename == NULL)
+       return NULL;
+
    /* Read first line and check that it is text */
    result = gnome_vfs_open (&handle, filename, GNOME_VFS_OPEN_READ);
    if (result != GNOME_VFS_OK) {
@@ -333,107 +339,95 @@ log_stats_new (char *filename, gboolean show_error)
    
    found_space = g_strstr_len (buff, 1024, " ");
    if (found_space == NULL) {
-	   if (show_error) {
-		   g_snprintf (buff, sizeof (buff), _("%s not a log file."), filename);
-		   ShowErrMessage (NULL, error_main, buff);
-	   }
        gnome_vfs_file_info_unref (info);
 	   return NULL;
    }
    
-   lstats = g_new (LogStats, 1);   
-   lstats->mtime = info->mtime;
-   lstats->size = info->size;
+   stats = g_new (LogStats, 1);   
+   stats->file_time = info->mtime;
+   stats->file_size = info->size;
    gnome_vfs_file_info_unref (info);
 
-   return (lstats);
+   return (stats);
 }
 
 Log *
 log_open (char *filename, gboolean show_error)
 {
-   Log *log;
-   LogLine line;
-   char *buffer;
-   char **buffer_lines;
-   char *display_name = NULL;
-   LogStats *stats;
-   int i, j;
-   GList *days;
-   Day *day;
-   GError *error;
+   char *buffer, *zipped_name= NULL, *error_message;
    GnomeVFSResult result;
-   int size;
-   gboolean has_dates;
+   LogStats *stats;
+   gboolean opened = TRUE;
+   int i, size;
+   GList *days;
+   Log *log;
    
-   if (file_exist (filename, show_error) == FALSE)
-	   return NULL;
-
-   if (file_is_zipped (filename)) {
-	   display_name = filename;
-	   filename = g_strdup_printf ("%s#gzip:", display_name);
-   }   
-
    stats = log_stats_new (filename, show_error);
-   if (stats == NULL)
-       return NULL;
-
-   result = gnome_vfs_read_entire_file (filename, &size, &buffer);
-   if (result != GNOME_VFS_OK) {
-	   ShowErrMessage (NULL, error_main, _("Unable to open logfile!\n"));
-	   return NULL;
+   if (stats == NULL) {
+       if (file_is_zipped (filename)) {    
+           zipped_name = g_strdup_printf ("%s#gzip:", filename);
+           stats = log_stats_new (filename, show_error);
+           if (stats == NULL) {
+               opened = FALSE;
+           }
+       } else
+           opened = FALSE;
+   }
+   
+   if (opened == FALSE) {
+       error_message = g_strdup_printf (_("%s is not a log file."), filename);
+       goto error;
    }
 
    log = g_new0 (Log, 1);   
    if (log == NULL) {
-	   ShowErrMessage (NULL, error_main, _("Not enough memory!\n"));
-	   return NULL;
+       error_message = g_strdup (_("Not enough memory."));
+       goto error;
    }
-   log->name = g_strdup (filename);
-   log->display_name = display_name;
-   if (display_name)
-	   g_free (filename);
 
-   buffer_lines = g_strsplit (buffer, "\n", -1);
-   g_free (buffer);
+   if (!zipped_name) {
+       log->name = g_strdup (filename);
+   } else {
+       log->name = zipped_name;
+       log->display_name = g_strdup (filename);
+   }
 
-   for (i=0; buffer_lines[i+1] != NULL; i++);
-   log->total_lines = i;
+   result = gnome_vfs_read_entire_file (log->name, &size, &buffer);
+   buffer[size-1] = 0;
+   if (result != GNOME_VFS_OK) {
+       error_message = g_strdup_printf (_("%s cannot be opened."), log->name);
+       goto error;
+   }
+
+   if (g_get_charset (NULL) == FALSE) {
+       char *buffer2;
+       buffer2 = locale_to_utf8 (buffer);
+       g_free (buffer);
+       buffer = buffer2;
+   }
+
+   log->lines = g_strsplit (buffer, "\n", -1);
+   g_free (buffer);   
+
+   log->total_lines = g_strv_length (log->lines);
    log->displayed_lines = 0;
    log->first_time = TRUE;
-   log->lstats = stats;
+   log->stats = stats;
+   log->model = NULL;
+   log->filter = NULL;
+   log->mon_offset = 0;
+   log->bold_rows_list = NULL;
 
-   /* Read the dates */
    /* A log without dates will return NULL */
-   log->days = log_read_dates (buffer_lines, log->lstats->mtime);
-   has_dates = (log->days != NULL);
-
-   /* Parse the lines */
-   log->lines = g_array_sized_new (FALSE, FALSE, sizeof (LogLine), log->total_lines);
-   if (!has_dates) {
-       for (i = 0; i < log->total_lines; i++) {
-           logline_fill_from_string (&line, buffer_lines[i], has_dates);
-           log->lines = g_array_append_val (log->lines, line);
-       }
-   } else {
-       for (days = log->days; days != NULL; days = g_list_next (days)) {
-           day = days->data;
-           for (i = day->first_line; i <= day->last_line; i++) {
-               logline_fill_from_string (&line, buffer_lines[i], has_dates);
-               log->lines = g_array_append_val (log->lines, line);
-           }
-       }
-   }
-   g_strfreev (buffer_lines);
+   log->days = log_read_dates (log->lines, log->stats->file_time);
 
    /* Check for older versions of the log */
    log->versions = 0;
    log->current_version = 0;
    log->parent_log = NULL;
-   log->mon_offset = size;
    for (i=1; i<5; i++) {
        gchar *older_name;
-       older_name = g_strdup_printf ("%s.%d", filename, i);
+       older_name = g_strdup_printf ("%s.%d", log->name, i);
        log->older_logs[i] = log_open (older_name, FALSE);
        g_free (older_name);
        if (log->older_logs[i] != NULL) {
@@ -446,28 +440,42 @@ log_open (char *filename, gboolean show_error)
    }
 
    return log;
+
+   /* Error catching */
+
+ error:
+
+   if (error_message && show_error) {
+       error_dialog_show (NULL, error_main, error_message);
+       g_free (error_message);
+   }
+   return NULL;
+   
 }
 
 static void
 log_add_lines (Log *log, gchar *buffer)
 {
-  char **buffer_lines;
-  int i;
-  LogLine line;
+  char *old_buffer, *new_buffer;
 
-  g_return_if_fail (log != NULL);
-  g_return_if_fail (buffer != NULL);
+  g_assert (log != NULL);
+  g_assert (buffer != NULL);
 
-  buffer_lines = g_strsplit (buffer, "\n", -1);
+  old_buffer = g_strjoinv ("\n", log->lines);
+  new_buffer = g_strconcat (old_buffer, "\n", buffer, NULL);
+  g_free (old_buffer);
   
-  for (i=0; buffer_lines[i+1] != NULL; i++) {
-      logline_fill_from_string (&line, buffer_lines [i], (log->days != NULL));
-      log->lines = g_array_append_val (log->lines, line);
-      log->total_lines ++;
-  }
-  g_strfreev (buffer_lines);
- 
-  return;
+  g_strfreev (log->lines);
+  log->lines = g_strsplit (new_buffer, "\n", -1);
+  g_free (new_buffer);
+
+  log->total_lines = g_strv_length (log->lines);
+}
+
+void log_stats_reload (Log *log)
+{
+  g_free (log->stats);
+  log->stats = log_stats_new (log->name, TRUE);
 }
 
 /* log_read_new_lines */
@@ -475,29 +483,65 @@ log_add_lines (Log *log, gchar *buffer)
 gboolean 
 log_read_new_lines (Log *log)
 {
-	GnomeVFSResult result;
-	gchar *buffer;
-    GnomeVFSFileSize newsize, read;
-    GnomeVFSFileOffset size;
-
-	g_return_val_if_fail (log!=NULL, FALSE);
+    GnomeVFSResult result;
+    gchar *buffer;
+    GnomeVFSFileSize newfilesize, read;
+    int size, newsize;
+    
+    g_return_val_if_fail (log!=NULL, FALSE);
     
     result = gnome_vfs_seek (log->mon_file_handle, GNOME_VFS_SEEK_END, 0L);
-	result = gnome_vfs_tell (log->mon_file_handle, &newsize);
-    size = log->mon_offset;
+    result = gnome_vfs_tell (log->mon_file_handle, &newfilesize);
+    size = (int) log->mon_offset;
+    newsize = (int) newfilesize;
     
-	if (newsize > log->mon_offset) {
-        buffer = g_malloc (newsize-size);
-        result = gnome_vfs_seek (log->mon_file_handle, GNOME_VFS_SEEK_START, size);
-        result = gnome_vfs_read (log->mon_file_handle, buffer, newsize-size, &read);    
-        log->mon_offset = newsize;
-        
-        log_add_lines (log, buffer);
-        g_free (buffer);
-
-        return TRUE;
-	}
+    if (newsize > size) {
+      buffer = g_malloc (newsize - size);
+      if (!buffer)
 	return FALSE;
+
+      result = gnome_vfs_seek (log->mon_file_handle, GNOME_VFS_SEEK_START, size);
+      if (result != GNOME_VFS_OK)
+	return FALSE;
+
+      result = gnome_vfs_read (log->mon_file_handle, buffer, newsize-size, &read);
+      if (result != GNOME_VFS_OK)
+	return FALSE;
+
+      buffer [newsize-size-1] = 0;
+      log->mon_offset = newsize;      
+      log_add_lines (log, buffer);
+      g_free (buffer);
+      
+      log_stats_reload (log);
+      return TRUE;
+    }
+    return FALSE;
+}
+
+/* 
+   log_unbold is called by a g_timeout
+   set in loglist_bold_log 
+*/
+
+gboolean
+log_unbold (gpointer data)
+{
+    LogviewWindow *logview;
+    LogList *list;
+    Log *log = data;
+
+    g_return_if_fail (log != NULL);
+
+    logview = log->window;
+
+    /* If the log to unbold is not displayed, still wait */
+    if (logview_get_active_log (logview) != log)
+        return TRUE;
+
+    list = logview_get_loglist (logview);
+    loglist_unbold_log (list, log);
+    return FALSE;
 }
 
 void
@@ -505,8 +549,7 @@ log_close (Log *log)
 {
    gint i;
    Day *day;
-   GList *days;
-   LogLine *line;
+   GSList *days;
 
    g_return_if_fail (log);
    
@@ -521,34 +564,54 @@ log_close (Log *log)
    }
 
    g_object_unref (log->model);
-
-   /* Free all used memory */
-   for (i = 0; i < log->total_lines; i++) {
-       line = &g_array_index (log->lines, LogLine, i);
-       g_free (line -> message);
-       
-       if (log->days != NULL) {
-           g_free (line -> hostname);
-           g_free (line -> process);
-       }
-   }
-   g_array_free (log->lines, TRUE);
+   g_strfreev (log->lines);
 
    if (log->days != NULL) {
-       for (days = log->days; days != NULL; days = g_list_next (days)) {
+       for (days = log->days; days != NULL; days = g_slist_next (days)) {
            day = days->data;
            g_date_free (day->date);
            gtk_tree_path_free (day->path);
            g_free (day);
        }
-       g_list_free (log->days);
+       g_slist_free (log->days);
        log->days = NULL;
    }
    
-   g_free (log->lstats);
-   gtk_tree_path_free (log->current_path);
-   log->current_path = NULL;
+   g_free (log->stats);
+
+   if (log->selected_range.first)
+     gtk_tree_path_free (log->selected_range.first);
+   if (log->selected_range.last)
+     gtk_tree_path_free (log->selected_range.last);
+   if (log->visible_first)
+     gtk_tree_path_free (log->visible_first);
 
    g_free (log);
    return;
 }
+
+gchar *
+log_extract_filename (Log *log)
+{
+  GnomeVFSURI *uri;
+  gchar *filename;
+
+  uri = gnome_vfs_uri_new (log->name);
+  filename = gnome_vfs_uri_extract_short_name (uri);
+  gnome_vfs_uri_unref (uri);
+
+  return filename;
+}
+
+gchar *
+log_extract_dirname (Log *log)
+{
+  GnomeVFSURI *uri;
+  gchar *dirname;
+
+  uri = gnome_vfs_uri_new (log->name);
+  dirname = gnome_vfs_uri_extract_dirname (uri);
+  gnome_vfs_uri_unref (uri);
+
+  return dirname;
+}
