@@ -174,13 +174,3 @@ date_get_string (GDate *date)
    
    return utf8;
 }
-
-void widget_set_font (GtkWidget *widget, gchar *font_name)
-{
-   PangoFontDescription *fontdesc;
-
-   fontdesc = pango_font_description_from_string (font_name);
-   if (pango_font_description_get_family (fontdesc) != NULL)
-       gtk_widget_modify_font (widget, fontdesc);
-   g_free (fontdesc);
-}
